<?php

/**
     * Admin functionality for FortGuard
     * @link https://fortguard.io/
     * @since 1.0.0
     * @package FortGuard
     * @subpackage FortGuard/inc
     * @author Nadil Karunarathna <nadil@hyvor.com>
 */

 namespace FortGuard;

 use FortGuard\FortGuard;

 class Admin {

    /**
     * @since 1.0.0
     * @var string $pluginIdentifier    Unique identifier for FortGuard Official Plugin
     */
    private $pluginIdentifier;

    /**
     * @since 1.0.0
     * @var string $pluginVersion    The version of this plugin
     */
    private $pluginVersion;

    /**
     * @since 1.0.0
     * @var string $nonce  The id for wp_create_nonce
     * Used to validate AJAX requests
     */
    private $nonce = 'fortguard';

    private $apiKey;


    /**
     * Initializes the class
     * @since 1.0.0
     */
    public function __construct($pluginIdentifier, $pluginVersion) {
        $this -> pluginIdentifier = $pluginIdentifier;
        $this -> pluginVersion = $pluginVersion;
        $this -> apiKey = FortGuard::getAPIKey();
        $this -> iconSpan = '<i class="ab-icon dashicons-shield"></i>';
    }

    /**
     * Adds CSS styles to the admin panel
     * 
     * @since 1.0.0
     */
    public function addStyles() {
        wp_enqueue_style(
            $this -> pluginIdentifier . '-admin',
            FORTGUARD_DIR_URL . 'static/admin.css',
            array(),
            $this -> pluginVersion,
            'all'
        );
    }

    /**
     * Adds JavaScript to the admin panel
     * 
     * @since 1.0.0
     */
    public function addScripts() {
        wp_enqueue_script(
            $this -> pluginIdentifier . '-admin',
            FORTGUARD_DIR_URL . 'static/admin.js',
            array('jquery', 'jquery-ui-autocomplete'),
            $this -> pluginVersion,
            true
        );

        $adminOptions = array(
			'ajaxURL' => admin_url( 'admin-ajax.php' ),
			'nonce'   => wp_create_nonce( $this -> nonce ),
	    );
	    wp_localize_script('fortguard-admin', 'fortguardAdminOptions', $adminOptions);
    }

    /**
     * Creates the plugin section for FortGuard
     * 
     * @since 1.0.0
     */
    public function createMenu() {
        
        add_menu_page(
            'FortGuard Plugin',         // Page title
            'FortGuard',                // Menu title
            'manage_options',           // Capability required to see this page
            'fortguard',                // Menu slug
            array($this, 'createView'), // Function that outputs the content of the page
            'dashicons-admin-generic',  // Icon URL
            99                          // Position in the menu order
        );
    }

    /**
     * Displays the admin panel for FortGuard
     * 
     * @since 1.0.0
     */
    public function createView() {
        include_once FORTGUARD_DIR_PATH . 'html/admin.php';
    } 

    public function createLogTable() {
        global $wpdb;
        $table_name = $wpdb->prefix . 'fortguard_logs';

        $charset_collate = $wpdb->get_charset_collate();

        $sql = "CREATE TABLE $table_name (
            id bigint(20) unsigned NOT NULL AUTO_INCREMENT,
            time datetime DEFAULT '0000-00-00 00:00:00' NOT NULL,
            comment_id bigint(20) unsigned NOT NULL,
            fortguard_response text NOT NULL,
            PRIMARY KEY  (id)
        ) $charset_collate;";

        require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
        dbDelta($sql);
    }

    /**
     * AJAX handler for saving settings
     * 
     * @since 1.0.0
     */
    public function a_saveSettings() {

        if (!isset($_POST['security']) || !wp_verify_nonce(sanitize_text_field(wp_unslash($_POST['security'])), $this -> nonce))
            $this -> a_finish(false, array('message' => 'Invalid request'));   

        if (isset($_POST['apiKey'])) 
            FortGuard::setAPIKey(sanitize_text_field(wp_unslash($_POST['apiKey'])));

        if (isset($_POST['contentSpamToggle']))
            update_option(FortGuard::TOGGLE_CONTENT_SPAM_OPTION_NAME, sanitize_text_field(wp_unslash($_POST['contentSpamToggle'])));

        if (isset($_POST['spamScore'])) 
            update_option(FortGuard::SPAM_SCORE_OPTION_NAME, sanitize_text_field(wp_unslash($_POST['spamScore'])));

        if (isset($_POST['model'])) 
            update_option(FortGuard::MODEL_OPTION_NAME, sanitize_text_field(wp_unslash($_POST['model'])));

        if (isset($_POST['locationSpamToggle']))
            update_option(FortGuard::TOGGLE_LOCATION_SPAM_OPTION_NAME, sanitize_text_field(wp_unslash($_POST['locationSpamToggle'])));

        if (isset($_POST['countries']))
            update_option(FortGuard::COUNTRIES_OPTION_NAME, sanitize_text_field(wp_unslash($_POST['countries'])));

        if (isset($_POST['countryExcludeInclude']))
            update_option(FortGuard::COUNTRY_EXCLUDE_INCLUDE_OPTION_NAME, sanitize_text_field(wp_unslash($_POST['countryExcludeInclude'])));

        if (isset($_POST['languageSpamToggle']))
            update_option(FortGuard::TOGGLE_LANGUAGE_SPAM_OPTION_NAME, sanitize_text_field(wp_unslash($_POST['languageSpamToggle'])));

        if (isset($_POST['languages']))
            update_option(FortGuard::LANGUAGES_OPTION_NAME, sanitize_text_field(wp_unslash($_POST['languages'])));

        if (isset($_POST['languageExcludeInclude']))
            update_option(FortGuard::LANGUAGE_EXCLUDE_INCLUDE_OPTION_NAME, sanitize_text_field(wp_unslash($_POST['languageExcludeInclude'])));

        if (isset($_POST['sentimentSpamToggle']))
            update_option(FortGuard::TOGGLE_SENTIMENTS_OPTION_NAME, sanitize_text_field(wp_unslash($_POST['sentimentSpamToggle'])));

        if (isset($_POST['sentiments']))
            update_option(FortGuard::SENTIMENTS_OPTION_NAME, sanitize_text_field(wp_unslash($_POST['sentiments'])));

        $this -> a_finish();
    }

    public function a_finish($status = true, $data = []) {

        header('Content-Type: application/json');

        $return = array(
            'status' => $status
        );
        $data = (array) $data;
        $return = array_merge($return, $data);
        echo wp_json_encode($return);
        die;

    }

    public function callFortGuardApi($comment, $languageSpamToggle, $sentimentSpamToggle) {
        
        $response = wp_remote_post('https://fortguard.io/api/v0/check', array(
            'headers' => array(
                'Content-Type' => 'application/json',
                'X-API-KEY' => $this->apiKey,
            ),
            'body' => wp_json_encode(array(
                "content" => array(
                    "model" => FortGuard::getModel(),
                    "context" => get_the_title($comment->comment_post_ID),
                    "content" => $comment->comment_content,
                    "language_detection" => ($languageSpamToggle == 1) ? true : false,
                    "sentiment_analysis" => ($sentimentSpamToggle == 1) ? true : false,
                    "sentiments" => ($sentimentSpamToggle == 1) ? array_map('trim', explode(',', FortGuard::getSentiments())) : []
                ),
                "ip" => array(
                    // "ip" => '141.98.215.233'
                    "ip" => $comment->comment_author_IP
                )
            )),
        ));

        if (!is_wp_error($response) && $response['response']['code'] === 200) {
            $data = json_decode($response['body']);
            return $data;
        } else {
            return json_decode($response);
        }
    }

    public function callFortGuardIpApi($comment) {

        $response = wp_remote_post('https://fortguard.io/api/v0/check/ip', array(
            'headers' => array(
                'Content-Type' => 'application/json',
                'X-API-KEY' => $this->apiKey,
            ),
            'body' => wp_json_encode(array(
                "ip" => '141.98.215.233'
                // "ip" => $comment->comment_author_IP
            )),
        ));

        if (!is_wp_error($response) && $response['response']['code'] === 200) {
            $data = json_decode($response['body']);
            return $data;
        } else {
            return json_decode($response);
        }
    }
        
    public function callFortGuardContentApi($comment, $languageSpamToggle, $sentimentSpamToggle){
            
            $response = wp_remote_post('https://fortguard.io/api/v0/check/content', array(
                'headers' => array(
                    'Content-Type' => 'application/json',
                    'X-API-KEY' => $this->apiKey,
                ),
                'body' => wp_json_encode(array(
                    "model" => FortGuard::getModel(),
                    "context" => get_the_title($comment->comment_post_ID),
                    "content" => $comment->comment_content,
                    "language_detection" => ($languageSpamToggle == 1) ? true : false,
                    // "language_detection" => true,
                    "sentiment_analysis" => ($sentimentSpamToggle == 1) ? true : false,
                    "sentiments" => ($sentimentSpamToggle == 1) ? array_map('trim', explode(',', FortGuard::getSentiments())) : []
                )),
            ));
    
            if (!is_wp_error($response) && $response['response']['code'] === 200) {
                $data = json_decode($response['body']);
                return $data;
            } else {
                return json_decode($response);
            }
    }
        

    public function addReply($comment_id, $comment, $fortguardResponse) {

        if ($comment->comment_author !== 'Automated Response') {

            $reply_data = array(
                'comment_post_ID' => $comment->comment_post_ID,
                'comment_author' => 'Automated Response',
                'comment_author_email' => 'auto-reply@example.com',
                'comment_author_url' => '',
                'comment_content' => wp_json_encode($fortguardResponse),
                // 'comment_content' => wp_json_encode($dummyData),
                'comment_type' => 'comment',
                'comment_parent' => $comment_id,
                'user_id' => 0,
                'comment_author_IP' => $comment->comment_author_IP,
                'comment_agent' => 'AutoResponder',
                'comment_date' => current_time('mysql'),
                'comment_approved' => 1,
            );

            wp_insert_comment($reply_data);
        }
    }

    public function logResponse($comment_id, $fortguardResponse) {
        global $wpdb;
        $table_name = $wpdb->prefix . 'fortguard_logs';

        $wpdb->insert(
            $table_name,
            array(
                'time' => current_time('mysql'),
                'comment_id' => $comment_id,
                'fortguard_response' => $fortguardResponse
            )
        );
    }

    public function handleComments($comment_id, $comment) {
        
        $contentObject = null;
        $ipObject = null;

        $locationSpamToggle = get_option(FortGuard::TOGGLE_LOCATION_SPAM_OPTION_NAME);
        $contentSpamToggle = get_option(FortGuard::TOGGLE_CONTENT_SPAM_OPTION_NAME);
        $languageSpamToggle = get_option(FortGuard::TOGGLE_LANGUAGE_SPAM_OPTION_NAME);
        $sentimentSpamToggle = get_option(FortGuard::TOGGLE_SENTIMENTS_OPTION_NAME);

        if (($locationSpamToggle == 1) and ($contentSpamToggle == 0) and ($languageSpamToggle == 0) and ($sentimentSpamToggle == 0)) {
            $ipObject = $this->callFortGuardIpApi($comment);
            $this->logResponse($comment_id, wp_json_encode($ipObject));
            
            
        }
        elseif (($locationSpamToggle == 0) and (($contentSpamToggle == 1) or ($languageSpamToggle == 1) or ($sentimentSpamToggle == 1))) {
            $contentObject = $this->callFortGuardContentApi($comment, $languageSpamToggle, $sentimentSpamToggle);
            $this->logResponse($comment_id, wp_json_encode($contentObject));

        }
        else {
            $fortguardResponse = $this->callFortGuardApi($comment, $languageSpamToggle, $sentimentSpamToggle);
            $this->logResponse($comment_id, wp_json_encode($fortguardResponse));
            $ipObject = $fortguardResponse->ip;
            $contentObject = $fortguardResponse->content;
        }

        // $this->addReply($comment_id, $comment, wp_json_encode(array($contentObject, $ipObject)));

        if ($contentSpamToggle == 1) {

            $commentSpamScore = $contentObject->spam_score;

            if ($commentSpamScore >= FortGuard::getSpamScore())
                wp_spam_comment($comment_id);
        }

        if ($languageSpamToggle == 1) {

            $languages = array_map('trim', explode(',', FortGuard::getLanguages()));

            if (FortGuard::getLanguageExcludeInclude() === 'exclude' && in_array($contentObject->language_code, $languages)) 
                wp_spam_comment($comment_id);

            if (FortGuard::getLanguageExcludeInclude() === 'include' && !in_array($contentObject->language_code, $languages)) 
                wp_spam_comment($comment_id);
        }

        if ($locationSpamToggle == 1) {

            $countries = array_map('trim', explode(',', FortGuard::getCountries()));

            if (FortGuard::getCountryExcludeInclude() === 'exclude' && in_array($ipObject->location->country_code, $countries)) 
                wp_spam_comment($comment_id);

            if (FortGuard::getCountryExcludeInclude() === 'include' && !in_array($ipObject->location->country_code, $countries)) 
                wp_spam_comment($comment_id);
        }
    }
}