<?php

/**
 * The main class of the FortGuard Plugin for wordpress.
 * @link https://fortguard.io/
 * @since 1.0.0
 * @package FortGuard
 * @subpackage FortGuard/inc
 * @author Nadil Karunarathna <nadil@hyvor.com>
 */

namespace FortGuard;

class FortGuard {

    /**
     * @since 1.0.0
     * @var string	$pluginIdentifier 	Unique identifier for FortGuard Official Plugin
     */
    private $pluginIdentifier;

    /**
     * @since 1.0.0
     * @var string $pluginVersion 	The version of this plugin
     */
    private $pluginVersion;

    /**
     * @since 1.0.0
     * @var string API_KEY_OPTION_NAME  WP option name for the API key
     */
    const API_KEY_OPTION_NAME = 'fortguard_api_key';

    /**
     * @since 1.0.0
     * @var string TOGGLE_CONTENT_SPAM_OPTION_NAME  WP option name for the toggle content spam
     */
    const TOGGLE_CONTENT_SPAM_OPTION_NAME = 'fortguard_toggle_content_spam';

    /**
     * @since 1.0.0
     * @var string SPAM_SCORE_OPTION_NAME  WP option name for the spam score
     */
    const SPAM_SCORE_OPTION_NAME = 'fortguard_spam_score';

    /**
     * @since 1.0.0
     * @var string MODEL_OPTION_NAME  WP option name for the model
     */
    const MODEL_OPTION_NAME = 'fortguard_model';

    /**
     * @since 1.0.0
     * @var string TOGGLE_LANGUAGE_SPAM_OPTION_NAME  WP option name for the toggle language spam
     */    
    const TOGGLE_LANGUAGE_SPAM_OPTION_NAME = 'fortguard_toggle_language_spam';

    /**
     * @since 1.0.0
     * @var string LANGUAGES_OPTION_NAME  WP option name for the languages
     */
    const LANGUAGES_OPTION_NAME = 'fortguard_languages';

    /**
     * @since 1.0.0
     * @var string LANGUAGE_EXCLUDE_INCLUDE_OPTION_NAME  WP option name for the exclude/include
     */
    const LANGUAGE_EXCLUDE_INCLUDE_OPTION_NAME = 'fortguard_language_exclude_include';

    /**
     * @since 1.0.0
     * @var string TOGGLE_SENTIMENTS_OPTION_NAME  WP option name for the toggle sentiments
     */
    const TOGGLE_SENTIMENTS_OPTION_NAME = 'fortguard_toggle_sentiments';

    /**
     * @since 1.0.0
     * @var string SENTIMENTS_OPTION_NAME  WP option name for the sentiments
     */
    const SENTIMENTS_OPTION_NAME = 'fortguard_sentiments';

    /**
     * @since 1.0.0
     * @var string TOGGLE_LOCATION_SPAM_OPTION_NAME  WP option name for the toggle country spam
     */
    const TOGGLE_LOCATION_SPAM_OPTION_NAME = 'fortguard_toggle_location_spam';

    /**
     * @since 1.0.0
     * @var string COUNTRIES_OPTION_NAME  WP option name for the countries
     */
    const COUNTRIES_OPTION_NAME = 'fortguard_countries';

    /**
     * @since 1.0.0
     * @var string COUNTRY_EXCLUDE_INCLUDE_OPTION_NAME  WP option name for the exclude/include
     */
    const COUNTRY_EXCLUDE_INCLUDE_OPTION_NAME = 'fortguard_country_exclude_include';

    /**
     * Initializes the class
     * @since 1.0.0
     */
    public function __construct() {
        $this -> pluginIdentifier = FORTGUARD_IDENTIFIER;
        $this -> pluginVersion = FORTGUARD_VERSION;

        $this -> initHelpers();
        $this -> initAdminHooks();
    }


    /**
     * Initializes the helper classes
     * @since 1.0.0
     */
    private function initHelpers() {
        /**
         * Handle the admin functionality of the plugin
         */
        require_once FORTGUARD_DIR_PATH . 'inc/class-fortguard-admin.php';
    }

    /**
     * Initializes the admin hooks
     * @since 1.0.0
     */
    private function initAdminHooks() {
        // register_activation_hook(__FILE__, 'activate_fortguard_plugin');

        $admin = new Admin($this -> pluginIdentifier, $this -> pluginVersion);
        $admin->createLogTable();
        add_action('admin_enqueue_scripts', array($admin, 'addStyles'));
        add_action('admin_enqueue_scripts', array($admin, 'addScripts'));
        add_action('admin_menu', array($admin, 'createMenu'));
        add_action('wp_ajax_save_settings', array($admin, 'a_saveSettings'));
        add_action('wp_insert_comment', array($admin, 'handleComments'), 10, 2);
    }


    /**
     * Returns the API key for FortGuard
     * @return string|null
     */
    public static function getApiKey() {
        $ApiKey = get_option(self::API_KEY_OPTION_NAME);
        return $ApiKey ? $ApiKey : null;
    }

    /**
     * Sets the API key for FortGuard
     * @param string $APIKey The API key to set
     */
    public static function setApiKey($ApiKey) {
        update_option(self::API_KEY_OPTION_NAME, $ApiKey);
    }

    /**
     * Returns the model for FortGuard
     * @return string|null
     */
    public static function getModel() {
        $model = get_option(self::MODEL_OPTION_NAME);
        return $model ? $model : null;
    }

    /**
     * Sets the model for FortGuard
     * @param string $model The model to set
     */
    public static function setModel($model) {
        update_option(self::MODEL_OPTION_NAME, $model);
    }

    /**
     * Returns the toggle content spam for FortGuard
     * @return int
     */
    public static function getContentSpamToggle() {
        $contentSpamToggle = get_option(self::TOGGLE_CONTENT_SPAM_OPTION_NAME);
        return ($contentSpamToggle == 0) ? 0 : 1;
    }

    /**
     * Sets the toggle content spam for FortGuard
     * @param int $contentSpamToggle The toggle content spam to set
     */
    public static function setContentSpamToggle($contentSpamToggle) {
        update_option(self::TOGGLE_CONTENT_SPAM_OPTION_NAME, $contentSpamToggle);
    }

    /**
     * Returns the spam score for FortGuard
     * @return int|null
     */
    public static function getSpamScore() {
        $spamScore = get_option(self::SPAM_SCORE_OPTION_NAME);
        return $spamScore ? (int) $spamScore : null;
    }

    /**
     * Sets the spam score for FortGuard
     * @param int $spamScore The spam score to set
     */
    public static function setSpamScore($spamScore) {
        update_option(self::SPAM_SCORE_OPTION_NAME, $spamScore);
    }

    /**
     * Returns the toggle language spam for FortGuard
     * @return int
     */
    public static function getSentimentSpamToggle() {
        $sentimentSpamToggle = get_option(self::TOGGLE_SENTIMENTS_OPTION_NAME);
        return ($sentimentSpamToggle == 0) ? 0 : 1;
    }

    /**
     * Sets the toggle sentiments for FortGuard
     * @param int $sentimentSpamToggle The toggle sentiments to set
     */    
    public static function setSentimentSpamToggle($sentimentSpamToggle) {
        update_option(self::TOGGLE_SENTIMENTS_OPTION_NAME, $sentimentSpamToggle);
    }

    /**
     * Returns the sentiments for FortGuard
     * @return string|null
     */
    public static function getSentiments() {
        $sentiments = get_option(self::SENTIMENTS_OPTION_NAME);
        return $sentiments ? $sentiments : null;
    }

    /**
     * Sets the sentiments for FortGuard
     * @param string $sentiments The sentiments to set
     */
    public static function setSentiments($sentiments) {
        update_option(self::SENTIMENTS_OPTION_NAME, $sentiments);
    }

    /**
     * Returns the toggle country spam for FortGuard
     * @return int
     */
    public static function getLocationSpamToggle() {
        $locationSpamToggle = get_option(self::TOGGLE_LOCATION_SPAM_OPTION_NAME);
        return ($locationSpamToggle == 0) ? 0 : 1;
    }

    /**
     * Sets the toggle country spam for FortGuard
     * @param int $locationSpamToggle The toggle country spam to set
     */
    public static function setLocationSpamToggle($locationSpamToggle) {
        update_option(self::TOGGLE_LOCATION_SPAM_OPTION_NAME, $locationSpamToggle);
    }

    /**
     * Returns the countries for FortGuard
     * @return string|null
     */
    public static function getCountries() {
        $countries = get_option(self::COUNTRIES_OPTION_NAME);
        return $countries ? $countries : null;
    }

    /**
     * Sets the countries for FortGuard
     * @param string $countries The countries to set
     */
    public static function setCountries($countries) {
        update_option(self::COUNTRIES_OPTION_NAME, $countries);
    }

    /**
     * Returns the country exclude/include for FortGuard
     * @return string|null
     */
    public static function getCountryExcludeInclude() {
        $countryExcludeInclude = get_option(self::COUNTRY_EXCLUDE_INCLUDE_OPTION_NAME);
        return $countryExcludeInclude ? $countryExcludeInclude : null;
    }

    /**
     * Sets the country exclude/include for FortGuard
     * @param string $countryExcludeInclude The country exclude/include to set
     */
    public static function setCountryExcludeInclude($countryExcludeInclude) {
        update_option(self::COUNTRY_EXCLUDE_INCLUDE_OPTION_NAME, $countryExcludeInclude);
    }

    /**
     * Returns the toggle language spam for FortGuard
     * @return int
     */
    public static function getLanguageSpamToggle() {
        $languageSpamToggle = get_option(self::TOGGLE_LANGUAGE_SPAM_OPTION_NAME);
        return ($languageSpamToggle == 0) ? 0 : 1;
    }

    /**
     * Sets the toggle language spam for FortGuard
     * @param int $languageSpamToggle The toggle language spam to set
     */
    public static function setLanguageSpamToggle($languageSpamToggle) {
        update_option(self::TOGGLE_LANGUAGE_SPAM_OPTION_NAME, $languageSpamToggle);
    }

    /**
     * Returns the languages for FortGuard
     * @return string|null
     */
    public static function getLanguages() {
        $languages = get_option(self::LANGUAGES_OPTION_NAME);
        return $languages ? $languages : null;
    }

    /**
     * Sets the languages for FortGuard
     * @param string $languages The languages to set
     */
    public static function setLanguages($languages) {
        update_option(self::LANGUAGES_OPTION_NAME, $languages);
    }

    /**
     * Returns the language exclude/include for FortGuard
     * @return string|null
     */
    public static function getLanguageExcludeInclude() {
        $languageExcludeInclude = get_option(self::LANGUAGE_EXCLUDE_INCLUDE_OPTION_NAME);
        return $languageExcludeInclude ? $languageExcludeInclude : null;
    }
    
    /**
     * Sets the language exclude/include for FortGuard
     * @param string $languageExcludeInclude The language exclude/include to set
     */
    public static function setLanguageExcludeInclude($languageExcludeInclude) {
        update_option(self::LANGUAGE_EXCLUDE_INCLUDE_OPTION_NAME, $languageExcludeInclude);
    }
}